package qt.demo

import com.trolltech.qt.gui._
import com.trolltech.qt.core._
import com.trolltech.qt.core.Qt._

object CustomWindow {

  //=======================// 全局样式 //=======================//

  val globalStyle = """
* { font-family: Microsoft Yahei; font-size: 12px; color: #333; }
#mainWindow { border: 32px solid #ccc; border-image: url(classpath:qt/demo/resource/window.png) 32 stretch; }
#title { font-size: 13px; font-weight: bold; color: #000; }
#body { border: 1px solid #ccc; }
#closeBtn { border: 0; background: none; font-weight: bold; color: red; }
"""

  //=======================// 主窗体 //=======================//
  // 其实我们可以将这个Frame作为一个单独类封装一下
  class DemoFrame extends QFrame {

    this.setObjectName("mainWindow")
    // 以下为窗体展现定制，应该在show之前调用
    // show以后再调用，会令窗体crash，你需要再次show
    this.setWindowFlags(WindowType.FramelessWindowHint)
    this.setAttribute(WidgetAttribute.WA_TranslucentBackground, true)

    val frameLayout = new QVBoxLayout(this) {
      this.setMargin(0)  // Layout四边的margin，当然你也可以用setContentsMargin来设定四边的具体值
      this.setSpacing(0) // 设定元件之间的间距
    }

    private def frame = this

    //=======================// 标题栏 //=======================//

    val titleLayout = new QHBoxLayout {
      setSpacing(5)
      setAlignment(new Alignment(AlignmentFlag.AlignLeft, AlignmentFlag.AlignVCenter))
    }

    // 标题的元素构成
    val titleIcon = new QLabel {
      this.setPixmap(getIcon.pixmap(18))
      // 指定他的具体宽高
      this.setFixedSize(20, 20)
      // 让该容器按照该宽高占位
      this.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
    }
    val titleText = new QLabel("这是一个标题栏") {
      this.setObjectName("title")
      // 只指定具体高度
      this.setFixedHeight(20)
      // 他的水平方向会100%的撑开，垂直方向按指定的高度占位
      this.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
      // 我们从这里注册title拖动
      // 其实最好是title首先是个QWidget，那么我们可以对整个title进行拖动，请原谅我的懒惰
      val dragPosition = new QPoint()
      // 鼠标单击放下
      override def mousePressEvent(event: QMouseEvent) {
        if (event.button() == MouseButton.LeftButton) {
          val topLeft = frame.frameGeometry.topLeft
          dragPosition.setX(event.globalPos.x - topLeft.x)
          dragPosition.setY(event.globalPos.y - topLeft.y)
          event.accept
        }
      }

      override def mouseMoveEvent(event: QMouseEvent) {
        if (event.buttons().isSet(MouseButton.LeftButton)) {
          val topLeft = frame.frameGeometry.topLeft
          val p = new QPoint(event.globalPos().x() - dragPosition.x,
                             event.globalPos().y() - dragPosition.y)
          frame.move(p)
          event.accept
        }
      }
    }

    val closeBtn = new QPushButton("X")
    closeBtn.setObjectName("closeBtn")
    closeBtn.setFixedSize(15, 15)
    // 其实这里可以写成 this.clicked.connect(QApplication.instance, "quit()")
    // close表示关闭frame窗口，quit则是整个程序退出
    // qt内部会做调整，当你close窗口，又没有其他窗口在show或者准备show，他也会自动进行quit
    closeBtn.clicked.connect(this, "close()")

    override def showEvent(event: QShowEvent) {
      val g = this.frameGeometry
      val (w, h) = (g.width + 70, g.height + 70)
      this.setFixedSize(w, h) // 根据样式重设大小
      closeBtn.move(w - 45, 30)
    }

    //=======================// body部分 //=======================//

    val body = {
      val widget = new QWidget
      widget.setObjectName("body")
      widget.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
      widget
    }

    //=======================// foot部分 //=======================//

    val foot = new QLabel("状态栏") {
      this.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
      this.setAlignment(new Alignment(AlignmentFlag.AlignRight, AlignmentFlag.AlignBottom))
    }

    this.init()

    def init() {
      titleLayout.addWidget(titleIcon)
      titleLayout.addWidget(titleText)
      frameLayout.addLayout(titleLayout)
      frameLayout.addWidget(body)
      frameLayout.addWidget(foot)
      closeBtn.setParent(this) // closeBtn最后填充，因为他不在布局模式中。
    }

    def getIcon = frame.style.standardIcon(QStyle.StandardPixmap.SP_MessageBoxInformation)
  }

  def app = QApplication.instance

  lazy val frame = new DemoFrame

  def main(args: Array[String]): Unit = {
    QApplication.initialize(args)
    app.setStyleSheet(globalStyle)
    frame.resize(300, 200) // 我们可以自由的设定这个部件的大小了，调整元件的大小被我们送去showEvent里面去了
    frame.show()
    QApplication.exec
  }
}
